const assert = require("assert");
module.exports = function queueify(func, guardTime, maxOnce = 1, checkError){
  assert(func && maxOnce, "too few arguments");
  assert(guardTime == +guardTime, "guardTime is not a number");
  assert(typeof func == "function", "func is not a function");
  assert(maxOnce % 1 == 0, "fractal maxOnce");
  assert(maxOnce > 0, "maxOnce is not positive");

  let edge = Promise.resolve(), nextAt = 0, queue = [];
  return (...args) => new Promise((res, rej) => {
    function addTimeWait(){
      if(guardTime <= 0){return;}
      edge = edge
        .then(()=>new Promise(res => setTimeout(res, nextAt - Date.now())))
        .then(()=>nextAt = Date.now() + guardTime);
    }

    if(maxOnce == 1){
      addTimeWait();
      edge = edge
        .then(()=>func(...args))
        .then(res).catch(rej);
      return;
    }

    const isOverflow = queue.length % maxOnce == 0;
    queue.push([args, res, rej]);
    if(!isOverflow){return;}
    addTimeWait();

    edge = edge.then(async ()=>{
      const portion = queue.splice(0, maxOnce);
      try{
        const results = await func(portion.map(e=>e[0]));
        portion.forEach((e, i)=>{
          const result = results[i],
            args = portion[i][0],
            isErrorResult = checkError && checkError(result, args);

          if(isErrorResult){
            e[2](isErrorResult)
          }else{
            e[1](result)
          }
        });
      }catch(err){
        portion.forEach(e=>e[2](err))
      }
    });
  });
};
