const assert = require("assert"),
  queueify = require(`${__dirname}/`);

describe('Queueify', ()=>{
  let result = [];
  const guard = 300,
    func = x => result.push([x, Date.now()]),
    funcQ = queueify(func, guard);

  it('should make sequental runs with 99% accurate timings', async ()=>{
    await funcQ(1);
    funcQ(2).then(()=>result.push(3))
    funcQ(4).then(()=>result.push(5))
    await funcQ(6);

    assert(result[0][0] == 1 &&
      result[1][0] == 2 &&
      result[2] == 3 &&
      result[3][0] == 4 &&
      result[4] == 5 &&
      result[5][0] == 6 &&
      result[0][1] < result[1][1] &&
      result[1][1] < result[3][1] &&
      result[3][1] < result[5][1] &&
      Math.min(result[1][1] - result[0][1],
        result[3][1] - result[1][1],
        result[5][1] - result[3][1]) > 0.99*guard
    );

    console.log(result);
  });
});

function diffRate(a, b, rate){
  assert(a>0);
  assert(b>0);
  return 1 - ( 2*Math.abs(a-b)/(a+b) ) > rate
}

describe('Queueify with maxOnce and no wait ', ()=>{
  it("should make sequences with len maxOnce and 99% accurate timings", async ()=>{
    let result = [];
    const func = x => {
        result.push([x, Date.now()]);
        return x+" "+x;
      },
      funcMany = arr=>arr.map(func)
      funcQ = queueify(funcMany, 30, 4);

    funcQ(1,123);
    funcQ(2);
    const a = await funcQ(3);
    funcQ(4);
    funcQ(5);
    funcQ(6);
    funcQ(7);
    funcQ(8);
    funcQ(9);
    const b = await funcQ(10);
    funcQ(11);
    const c = await funcQ(12);

    assert.equal(a, "3 3");
    assert.equal(b, "10 10");
    assert.equal(c, "12 12");
    assert(diffRate(result[0][1], result[1][1], 0.99));
    assert(diffRate(result[1][1], result[2][1], 0.99));

    assert(diffRate(result[3][1], result[4][1], 0.99));
    assert(diffRate(result[4][1], result[5][1], 0.99));
    assert(diffRate(result[5][1], result[6][1], 0.99));

    assert(diffRate(result[7][1], result[8][1], 0.99));
    assert(diffRate(result[8][1], result[9][1], 0.99));

    assert(diffRate(result[10][1], result[11][1], 0.99));

    assert.deepEqual(result[0][0], [1,123]);
    for(let i=1;i<12;i++){
      assert.deepEqual(result[i][0], [i+1]);
    }

    console.log(result);
  });
});
